function [OutCellArray, Outconfig]=Coordinates2Map(InCellArray, Inconfig)
%Coordinates2Map allows to converts a non-uniform(scattered) dataset to the map type data.
%The data is assumed to be 3 columns, in X,Y,Z order.
%
%Inputs: Coordinates
%Outputs: Map
%
%Author: Won Hyun Park
%Email: nova8028@gmail.com, whpark@optics.arizona.edu
%
% Version history:
% 9/28/10 - Version 1.0
% 8/20/11 - Version 1.1 Dae Wook Kim letter2dwk@hotmail.com (Modified
% Inputs and Outputs description) 
% 11/29/11 - Version 1.2 Benjamin Lewis blewis@email.arizona.edu (Modified
% for new data type definitions)

OutCellArray{1}.type='map';

if size(InCellArray{1}.data,2)~=3
    errordlg('Error: Data is expected to have 3 columns!','Coordinates2Map Error','modal');
    OutCellArray={};
    return;
end

labels=InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Column Label'),2};

labels=regexp(labels,', ','split');

if ~strcmpi(labels{1},'x')||~strcmpi(labels{2},'y')||~strcmpi(labels{3},'z')
    b=questdlg('Warning: Column Labels are not X, Y, Z.  Data is assumed to be in this order.  Continue?' ...
        ,'Coordinates2Map Error','Yes','Cancel','Yes');
    if ~strcmpi(b,'Yes')
        OutCellArray={};
        return;
    end
end


units=InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Column unit'),2};

units=regexp(units,', ','split');

x_unit=units{1};
y_unit=units{2};
z_unit=units{3};

Outconfig=Inconfig;
xs=Inconfig{strcmpi(Inconfig, 'X Size'),2};
ys=Inconfig{strcmpi(Inconfig, 'Y Size'),2};
xpnts=Inconfig{strcmpi(Inconfig, 'X Data Points'),2};
ypnts=Inconfig{strcmpi(Inconfig, 'Y Data Points'),2};

validinputs=0;
%Get inptuts
while ~validinputs
    prompt={'Input x size:','Input y size:', ...
        'Input the number of sampling points over x:', ...
        'Input the number of sampling points over y:'};
    name='Coordinates2Map';
    defaultanswer={xs,ys,xpnts,ypnts};
    answers=inputdlg(prompt,name,1,defaultanswer);
    if ~isempty(answers)
        temp_xs=str2double(answers{1});
        temp_ys=str2double(answers{2});
        temp_xpnts=str2double(answers{3});     
        temp_ypnts=str2double(answers{4}); 
        
        if ~isempty(temp_xs)&&~isempty(temp_ys)&&~isempty(temp_xpnts)&&~isempty(temp_ypnts) ...
                &&isnumeric(temp_xs)&&isnumeric(temp_ys)&&isnumeric(temp_xpnts)&&isnumeric(temp_ypnts)
            
            validinputs=1;             
            xs=temp_xs;
            ys=temp_ys;
            xpnts=temp_xpnts;
            ypnts=temp_ypnts;
        else
            msgbox('Please enter valid inputs','Invalid Inputs','warn','modal')
        end
    else
        OutCellArray={};
        return; %If answers is empty, user canceled
    end
end

Outconfig{strcmpi(Outconfig, 'X Size'),2}=num2str(xs);
Outconfig{strcmpi(Outconfig, 'Y Size'),2}=num2str(ys);
Outconfig{strcmpi(Outconfig, 'X Data Points'),2}=num2str(xpnts);
Outconfig{strcmpi(Outconfig, 'Y Data Points'),2}=num2str(ypnts);


OutCellArray{1}.header={'Date Created', ''};

OutCellArray{1}.header(end+1,:)={'X size', num2str(xs)};
OutCellArray{1}.header(end+1,:)={'Y size', num2str(ys)};

OutCellArray{1}.header(end+1,:)={'X unit', x_unit};
OutCellArray{1}.header(end+1,:)={'Y unit', y_unit};
OutCellArray{1}.header(end+1,:)={'Z unit', z_unit};



data = InCellArray{1}.data;

F = TriScatteredInterp(data(:,1),data(:,2),data(:,3));
tx = linspace(-xs/2,xs/2,xpnts);
ty = linspace(-ys/2,ys/2,ypnts);
[newx,newy] = meshgrid(tx,ty);
newz = F(newx,newy);
OutCellArray{1}.data=newz;

