function [OutCellArray, Outconfig]=GenDataConvert(InCellArray, Inconfig)
%GenDataConvert converts General data into other datatpes, and takes 
%inputs for any missing information needed for a dataset.  This allows a
%variety of data formats to imported into SAGUARO, independent of the
%source of the data.
%
%Inputs: General
%Outputs: Various types
%
%Author: Benjamin Lewis
%Email: blewis@email.arizona.edu
%
% Version history:
% 10/30/11 - Version 4
% 1/31/12 - Version 4.1

%Definitions%
typelist={'map','zernike','freqmap','coordinates','vectormap','mask'};

typetext.map='The map datatype is intended to store data such as surface OPD maps and interferograms.';
typetext.zernike='The zernike datatype stores zernike coefficients that represent a surface.';
typetext.freqmap='The freqmap datatype stores the Fourier transform of a surface.';
typetext.coordinates='The coordinates datatype stores cordinate tuples that represent a surface.';
typetext.vectormap='The vectormap datatype stores vector data at each data point.  This is inteded to store data such as surface gradient.';
typetext.mask='The mask datatype stores data that can be used to mask off a map.';

unitlist={'Custom','m','cm','mm','m','nm','1/mm','waves','Hz','rad','deg','arcmin','arcsec'};
%End Definitions%



Outconfig=Inconfig;

tempdatatype=Inconfig{strcmpi(Inconfig, 'Data Type Selection'),2};


f=figure('units','pixels','position',[520,380,534,312]);
set(f,'name','GenDataConvert_v4','HandleVisibility','off','menubar','none','NumberTitle','off','resize','off');

lb1=uicontrol('style','listbox','parent',f,'units','pixels','position',[18,70,240,224],'callback',@Lb1_callback);
ed1=uicontrol('style','edit','parent',f,'units','pixels','position',[276,70,240,224],'max',2);

okb=uicontrol('style','pushbutton','parent',f,'units','pixels','position',[178,40,70,22],'string','Ok','callback',@Okb_callback);
canb=uicontrol('style','pushbutton','parent',f,'units','pixels','position',[286,40,70,22],'string','Cancel','callback',@Canb_callback);

canceled=0;

datatype='';

set(lb1,'string',typelist);

set(lb1,'value',find(strcmpi(tempdatatype,typelist)));

uicontrol(lb1);

updatetext(lb1);

movegui(f,'center');

uiwait(f);

if ishandle(f), delete(f); end

if canceled
    OutCellArray={};
    return;
else
    Outconfig{strcmpi(Outconfig, 'Data Type Selection'),2}=datatype;
    
    f=strcat('template.', datatype);
    
    if exist(f, 'file')
        Ddtemplate=SAGUARO_dataset;
        Ddtemplate.load(f);
    else %No template was found for datatype
        errordlg('A template was not found for the datatype specified! Dataset will not be converted','Template not found', 'modal');
        
        OutCellArray={};
        return;
    end
    
    %Fill in header
    fieldslist=Ddtemplate.header(:,1);
    
    fieldslist(strcmpi(fieldslist, 'Date Created'))=[];
    
    switch datatype
        case 'coordinates'
            fdefault=Inconfig{strcmpi(Inconfig, 'coordinate flip'),2};
            
            qflip=questdlg('For Coordinate data, the data is expected to be in columns.  Does your data need to be transposed?', ...
                'GenDataConvert_v4','Yes','No',fdefault);
            
            Outconfig{strcmpi(Outconfig, 'coordinate flip'),2}=qflip;           
            
            switch qflip
                case 'Yes'
                    OutCellArray{1}.data=double(InCellArray{1}.data)';
                otherwise
                    OutCellArray{1}.data=double(InCellArray{1}.data);
            end
        otherwise
            OutCellArray{1}.data=double(InCellArray{1}.data);
    end
    
    
    OutCellArray{1}.type=datatype;
    OutCellArray{1}.header=Ddtemplate.header;
    
    %Get user inputs for remaining fields
    if ~isempty(fieldslist)
        
        defaults=Inconfig{strcmpi(Inconfig, [datatype ' defaults']),2};
        
        
        switch datatype
            case 'coordinates'
                tdefaults=regexp(defaults,'; ','split');
                
                tdefaults=tdefaults(1:end-1);
                
                numcols=size(OutCellArray{1}.data,2);
                
                
                for ii=1:numcols
                    tfieldslist{ii}=['Column ',num2str(ii),' unit'];
                end
                
                for ii=1:numcols
                    tfieldslist{ii+numcols}=['Column ',num2str(ii),' label'];
                end
                
                
                if numcols~=length(tdefaults)/2 %each layer has 2 entries, so this checks if the defaults save have the correct number of entries
                    defaults='';
                    
                    for ii=1:numcols
                        defaults=[defaults,'mm; '];
                    end
                    
                    for ii=1:numcols
                        defaults=[defaults,'X; '];
                    end
                end
                
                [tanswers,defaults]=BuildHeaderGui(tfieldslist,defaults,datatype);
                
                answers{1}=tanswers{1};
                answers{2}=tanswers{numcols+1};
                
                for ii=2:numcols
                    answers{1}=[answers{1},', ',tanswers{ii}];
                    answers{2}=[answers{2},', ',tanswers{numcols+ii}];
                end
                
            case 'vectormap'
                tdefaults=regexp(defaults,'; ','split');
                
                tdefaults=tdefaults(1:end-1);
                
                numlayers=size(OutCellArray{1}.data,3);
                
                
                for ii=1:numlayers
                    tfieldslist{ii}=['Layer ',num2str(ii),' unit'];
                end
                
                for ii=1:numlayers
                    tfieldslist{ii+numlayers}=['Layer ',num2str(ii),' label'];
                end
                
                
                if numlayers~=length(tdefaults)/2-4 %each layer has 2 entries, so this checks if the defaults save have the correct number of entries (4 entries at the start for other fields)
                    defaults='100; 100; mm; mm; ';
                    
                    for ii=1:numlayers
                        defaults=[defaults,'mm; '];
                    end
                    
                    for ii=1:numlayers
                        defaults=[defaults,'X; '];
                    end
                end
                
                [tanswers,defaults]=BuildHeaderGui([{'X size'},{'Y size'},{'X unit'},{'Y unit'},tfieldslist],defaults,datatype);
                
                fixed_answers=tanswers(1:4);
                
                tanswers(1:4)=[];
                
                answers{1}=tanswers{1};
                answers{2}=tanswers{numlayers+1};
                
                for ii=2:numlayers
                    answers{1}=[answers{1},', ',tanswers{ii}];
                    answers{2}=[answers{2},', ',tanswers{numlayers+ii}];
                end
                
                answers=[fixed_answers,answers];
                
            otherwise
                [answers,defaults]=BuildHeaderGui(fieldslist,defaults,datatype);
        end
        
        
        Outconfig{strcmpi(Outconfig, [datatype ' defaults']),2}=defaults;
        
        if isempty(answers), OutCellArray={}; return; end
        
        for ii=1:length(fieldslist) %Complete header
            if any(strcmpi(fieldslist(ii),Ddtemplate.header(:,1)))
                OutCellArray{1}.header{strcmpi(fieldslist(ii),Ddtemplate.header(:,1)),2}=answers{ii};
            end
        end
    end
    
    try
        eval(['[OutCellArray,Outconfig]=GenDataConvertHelper.' datatype 'Helper(OutCellArray,Outconfig);']);
    catch ex
        if strcmpi(ex.identifier,'MATLAB:subscripting:classHasNoPropertyOrMethod')
            %do nothing
        else
            rethrow(ex);
        end
    end
end


    function Okb_callback(~, ~)
        strings=get(lb1,'string');
        value=get(lb1,'value');
        datatype=strings{value};
        
        if ishandle(f), delete(f); end
    end

    function Canb_callback(~, ~)
        if ishandle(f), delete(f); end
        canceled=1;
    end

    function Lb1_callback(hObject, ~)
        updatetext(hObject);
    end

    function updatetext(hObject)
        strings=get(hObject,'string');
        value=get(hObject,'value');
        
        selection=strings{value};
        
        set(ed1,'string',textwrap(ed1,{typetext.(selection)}));
    end




    function [answers,defaultstring]=BuildHeaderGui(prompts,defaults,type)
        function defaults=parsedefault(str)
            splt=regexp(str,'; ','split');
            
            defaults=splt(1:end-1);
        end        
        
        function defaultstring=builddefaultstring(handles,isunits,isnum)
            defaultstring='';
            
            for qq=1:length(handles)
                if isnum(qq)
                    temp=get(handles{qq},'string');
                    
                    if isnumeric(str2num(temp))
                        defaultstring=[defaultstring,temp,'; '];
                    else
                        defaultstring=[defaultstring,'100; '];
                    end
                    
                elseif isunits(qq)
                    temp=get(handles{qq}.edit,'string');
                    
                    defaultstring=[defaultstring,temp,'; '];                    
                else
                    temp=get(handles{qq},'string');
                    
                    defaultstring=[defaultstring,temp,'; '];
                end
            end
        end
        
        function answers=getuseranswers(handles,isunits,isnum)
            for qq=1:length(handles)
                if isnum(qq)
                    temp=get(handles{qq},'string');
                    
                    if isnumeric(str2num(temp))
                        answers{qq}=temp;
                    else
                        answers{qq}='100';
                    end
                    
                elseif isunits(qq)
                    temp=get(handles{qq}.edit,'string');
                    
                    answers{qq}=temp;
                    
                else
                    temp=get(handles{qq},'string');
                    
                    answers{qq}=temp;
                end
            end
        end
              
        defaults=parsedefault(defaults);
               
        
        function handle=addstr_input(f,prompt,index,default)
            corner=lowerleftcorner(index);
            
            p=uipanel('units','pixels','units','pixels','position',[corner(1),corner(2),380,76],'parent',f,'BorderType','none');
            
            uicontrol('style','text','String',prompt,'units','pixels','position',[8,29,364,28],'parent',p,'fontsize',10);
            
            handle=uicontrol('style','edit','String',default,'units','pixels','position',[8,8,364,24],'parent',p,'fontsize',10);
        end
        
        function handle=addunit_input(f,prompt,index,default)
            corner=lowerleftcorner(index);
            
            p=uipanel('units','pixels','position',[corner(1),corner(2),380,76],'parent',f,'BorderType','none');
            
            uicontrol('style','text','String',prompt,'units','pixels','position',[8,29,364,28],'parent',p,'fontsize',10);
            
            handle.edit=uicontrol('style','edit','String',default,'units','pixels','position',[116,8,256,24],'parent',p,'fontsize',10);
            
            handle.dropdown=uicontrol('style','popupmenu','String',unitlist,'units','pixels','position',[8,8,100,24], ...
                'parent',p,'value',max([1,find(strcmpi(default,unitlist))]),'fontsize',9);
        end
        
        
        function corner=lowerleftcorner(index)
            corner=[10,46+86*(indexmax-index)];
        end
        
        
        function numeric_callback(hObject, ~)
            str=get(hObject, 'String');
            if ~isempty(str)
                if ~isnumeric(str2double(str))
                    set(hObject, 'String', '100');
                    errordlg('This input field expects numeric input!','GenDataConvert Error','Modal');
                end
            end
        end
        
        function units_callback(hObject, ~, edithandle)
            switch get(hObject,'Value')
                case 1 %Custom
                    set(edithandle,'Enable','On');
                otherwise %Defined Unit
                    set(edithandle,'Enable','Off');
                    
                    set(edithandle,'String',unitlist{get(hObject,'Value')});
            end
        end
                
        
        handles=[];
        
        isnum=zeros(length(prompts),1);
        isunits=zeros(length(prompts),1);
                
        indexmax=length(prompts);
        
        fig=figure('units','pixels','position',[200,200,430,min(700,46+86*indexmax)],'resize','off', ...
            'menubar','none','name',['GenDataConvert for ',upper(type)],'HandleVisibility','off', ...
            'NumberTitle','off','CloseRequestFcn',@closereq);
        
        movegui(fig,'center');
        
        figpanel=uipanel('parent',fig,'units','pixels','position',[0,0,431,min(700,46+86*indexmax)],'Clipping','on','BorderType','none');
        sliderpanel=uipanel('parent',figpanel,'units','pixels','position',[1,0,400,46+86*indexmax],'BorderType','none');

        s=uicontrol('Style','Slider','Parent',fig,'units','pixels','position',[407,10,20,min(700,46+86*indexmax)-20], ...
            'max',max((46+86*indexmax)-700,1),'min',0,'Value',0,'Callback',{@slider_callback1,sliderpanel});
 
        if 700>46+86*indexmax
            set(s,'enable','off');
        end
        
        function slider_callback1(hObject,~,panelhandel)
            pos=get(hObject,'Value');
            set(panelhandel,'Position',[1,-pos,400,min(700,46+86*indexmax)])
        end
        
        
        
        
        uicontrol('style','pushbutton','parent',sliderpanel,'units','pixels','position',[165,16,70,22],'string','Ok','callback',@Ok_callback);
        
        function Ok_callback(~, ~)
            closereq(fig,[]);
        end
        
        for rr=1:length(prompts)
            if ~isempty(strfind(lower(prompts{rr}),'size'))||any(strcmpi(prompts{rr},{'Obscuration Ratio','Radius'}))...
                    ||~isempty(strfind(lower(prompts{rr}),'Max Frequency'))
                
                isnum(rr)=1;
                
                handles{rr}=addstr_input(sliderpanel,prompts{rr},rr,defaults{rr});
                
                set(handles{rr},'callback',@numeric_callback);
                               
            elseif ~isempty(strfind(lower(prompts{rr}),'unit'))
                isunits(rr)=1;
                
                handles{rr}=addunit_input(sliderpanel,prompts{rr},rr,defaults{rr});
                
                set(handles{rr}.dropdown,'callback',{@units_callback,handles{rr}.edit});
                
                units_callback(handles{rr}.dropdown, [], handles{rr}.edit);
                
            else
                handles{rr}=addstr_input(sliderpanel,prompts{rr},rr,defaults{rr});
                               
            end
        end
        
        defaultstring='';
        answers={};
        
        uiwait(fig);
        
        
        function closereq(hObject,~)
            try
                defaultstring=builddefaultstring(handles,isunits,isnum);
                
                answers=getuseranswers(handles,isunits,isnum);
                
                answers=regexprep(answers,'\;','');
                
                pause(.1);
                
                delete(hObject);
            catch ex
                delete(hObject);
                
                rethrow(ex);
            end
        end
    end
end


