function [OutCellArray, Outconfig]=MapRotate(InCellArray, Inconfig)
%MapRotate rotates a map about the center of the map by a user-specified 
% angle. Units on x and y axes must be same due to possible map boundary 
% reshaping upon rotation. Note: Due to interpolation, multiple rotations
% can blur features in image. Maps with large aspect ratios suffer most.
%
%Inputs: Map
%Outputs: Map
%
%Author: Benjamin Lewis
%Email: blewis@email.arizona.edu
%
% Version history:
% 8/13/10 - Version 1
% 11/16/2011 - Version 1.1 - Greg Smith - allow for non-rectangular arrays

% use input as default output
Outconfig = Inconfig;
OutCellArray = InCellArray;

DegRot=Inconfig{strcmpi(Inconfig, 'Rotation (Degrees)'),2};

validinputs=0;

%Get inputs
while ~validinputs
    answers = inputdlg({'Rotation (degrees)'},'MapRotate',1,{DegRot});
    if ~isempty(answers)
        tempdeg=answers{1};              
        tempdeg=str2double(tempdeg);

        if ~isempty(tempdeg)&&isnumeric(tempdeg)
            validinputs=1;             
            DegRot=tempdeg;
        else
            msgbox('Please enter valid inputs','Invalid Inputs','warn','modal')
        end
    else
        OutCellArray={};
        return; %If answers is empty, user canceled
    end
end

Outconfig{strcmpi(Outconfig, 'Rotation (Degrees)'),2}=num2str(DegRot);

RadRot=DegRot*pi/180;

% extract data, header, dimensions, and min value
% because of reshaping with rotation, x and y units must be same
xSize=str2double(InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'X Size'),2});
ySize=str2double(InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Y Size'),2});
xUnit=InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'X Unit'),2};
yUnit=InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Y Unit'),2};
if (~strcmpi(xUnit,yUnit))
    msgbox('Axes must have the same dimensions. Please convert and try again. Calculation is aborting...','Invalid Inputs','warn','modal');
    Outconfig=Inconfig;
    OutCellArray=InCellArray;
    return
end
surfmap = InCellArray{1}.data;
[pointsY,pointsX] = size(surfmap);
minVal = min(min(surfmap));

% convert data to polar coordinates
x=linspace(-xSize/2, xSize/2, pointsX);
y=linspace(-ySize/2, ySize/2, pointsY);
[x,y]=meshgrid(x,y);
[theta, rho, z] = cart2pol(x,y,surfmap);

% rotate map
theta = theta+RadRot;

% convert to Cartesian coordinates and interpolate to new grid
[xnew,ynew,znew]=pol2cart(theta(:), rho(:), z(:));
F = TriScatteredInterp(xnew, ynew, znew,'linear');

% create new output grid
% find new bounding box extents by rotating existing bounding box
maxXRot=max(xnew(znew>=minVal)); % extract min/max values of new bounding box - eliminate all points with NaN
minXRot=min(xnew(znew>=minVal)); 
maxYRot=max(ynew(znew>=minVal));
minYRot=min(ynew(znew>=minVal));
xRot=linspace(minXRot, maxXRot, ceil((maxXRot-minXRot)/xSize*pointsX));
yRot=linspace(minYRot, maxYRot, ceil((maxYRot-minYRot)/ySize*pointsY));
[xRot,yRot]=meshgrid(xRot,yRot);

% evaluate interpolation grid over new bounding box
surfmap=F(xRot,yRot);

% update parameters for return
OutCellArray{1}.data=surfmap;
OutCellArray{1}.header{strcmpi(OutCellArray{1}.header, 'X Size'),2} = num2str(maxXRot-minXRot);
OutCellArray{1}.header{strcmpi(OutCellArray{1}.header, 'Y Size'),2} = num2str(maxYRot-minYRot);

end