function [OutCellArray, OutConfig]=UnitConvert(InCellArray, InConfig)
% UnitConvert allows data units to be changed to other units.
% Common unit conversions are provided with the program. Additional unit
% conversion formulae can be provided by the user as needed.
%
% Inputs: 
% Outputs:
%
% Author: Greg Smith
% Email: gsmith@optics.arizona.edu
%
% Version history:
% 1/13/2012 - Version 1 - G.S. - initial implementation

% use input as default output
OutConfig = InConfig;
OutCellArray = InCellArray;

% create basic GUI to query user for unit conversion.
figureWidth = 940;
figureHeight = 540;
defaultFontSize = 10.0;
figureHandle = figure(...
            'Tag','unitConvertGUI',...
            'Position',[200,200,figureWidth,figureHeight],...
            'MenuBar','none',...
            'ToolBar','none',...
            'Resize','on');%,...
            %'WindowStyle','modal');

% perform unit conversion based on data type
dataType = InCellArray{1}.type;
RECURSIONLIMIT = 5;

switch dataType
    % MAP AND FREQMAP DATA TYPE
    case {'map','freqmap'}
        % Read parameters and data
        if strcmpi(dataType,'map')
            SizeX=InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'X size'),2};
            SizeY=InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Y size'),2};
        else
            SizeX=InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'X Max Frequency'),2};
            SizeY=InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Y Max Frequency'),2};
        end
        DataZ=InCellArray{1,1}.data;
        UnitsX=ChangeUnitName(InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'X unit'),2},'safe');
        UnitsY=ChangeUnitName(InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Y unit'),2},'safe');
        UnitsZ=ChangeUnitName(InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Z unit'),2},'safe');
        
        % information common to all datasets
        figureArray.FontSize = defaultFontSize;
        figureArray.RecursionLimit = RECURSIONLIMIT;
        figureArray.dataType = dataType;
        
        % Create GUI elements for X-axis
        figureArray.Tag = 'x';
        figureArray.Position = [0.05,0.6,0.25,0.1];
        figureArray.UnitsInput = UnitsX;
        figureArray.DataInput = SizeX;
        [xUnitsOut, xPathsOut, ~]=CreateGuiUnitDisplay(figureHandle,figureArray,InConfig);
        
        % Create GUI elements for Y-axis
        figureArray.Tag = 'y';
        figureArray.Position = [0.35,0.6,0.25,0.1];
        figureArray.UnitsInput = UnitsY;
        figureArray.DataInput = SizeY;
        [yUnitsOut, yPathsOut, ~]=CreateGuiUnitDisplay(figureHandle,figureArray,InConfig);
        
        % Create GUI elements for height data (z)
        figureArray.Tag = 'z';
        figureArray.Position = [0.65,0.6,0.25,0.1];
        figureArray.UnitsInput = UnitsZ;
        figureArray.DataInput = DataZ;
        [zUnitsOut, zPathsOut, ~]=CreateGuiUnitDisplay(figureHandle,figureArray,InConfig);
        
        %Create GUI Button to signal processing of all data 
        uicontrol('Parent',figureHandle,...
            'Tag','convertDataButton',...
            'Style','pushbutton',...
            'Enable','on',...
            'FontSize',defaultFontSize+1,...
            'FontWeight','normal',...
            'TooltipString','Apply unit conversions to all data..',...
            'Units','normalized',...  % must precede anything which uses dimensions
            'Position',[0.375,0.3,0.2,0.1],...
            'String','Convert Units',...
            'UserData','0',...
            'Callback',@GUIConvertData_Callback);
        
        % Perform actual conversion only when ready to exit (if not running in automated testing mode)
        if isfield(OutCellArray{1,1}, 'AutomatedMode')
            % Testing mode only
            parameters = OutCellArray{1,1}.AutomatedMode;
            allHandles = guihandles;
            set(allHandles.xUnitPopup,'Value',parameters(1));   % have user "select" desired unit conversion
            GUIUnit_Callback(allHandles.xUnitPopup,[],figureHandle,'xUnitPopup','xUnit');
            set(allHandles.yUnitPopup,'Value',parameters(2));
            GUIUnit_Callback(allHandles.yUnitPopup,[],figureHandle,'yUnitPopup','yUnit');
            set(allHandles.zUnitPopup,'Value',parameters(3));
            GUIUnit_Callback(allHandles.zUnitPopup,[],figureHandle,'zUnitPopup','zUnit');
        else
            % Note: this wait interferes with automated testing, hence the check
            % to ensure the module isn't being run in automated mode.
            uiwait(figureHandle);
        end
        
        % Get conversion factors stored in main figure 'UserData'
        if ishandle(figureHandle),conversionFactors = get(figureHandle,'UserData');
        else return; % abort calculation without changes if user closes window
        end
        
        % convert X-axis unit information and scale
        convert.conversionFactor = conversionFactors.xUnit;
        convert.defaultLabel = UnitsX;
        convert.pathList = xPathsOut;
        convert.unitList = xUnitsOut;
        convert.dataIn = str2double(SizeX);
        if strcmpi(dataType,'map'), convert.dataLabel = 'X size'; else convert.dataLabel = 'X Max Frequency'; end
        [OutCellArray,OutConfig] = UpdateHeader(OutCellArray,OutConfig,convert); % update value
        convert.dataIn = '';    % use default
        convert.dataLabel = 'X unit';
        [OutCellArray,OutConfig] = UpdateHeader(OutCellArray,OutConfig,convert); % update unit label

        % convert Y-axis unit information and scale
        convert.conversionFactor = conversionFactors.yUnit;
        convert.defaultLabel = UnitsY;
        convert.pathList = yPathsOut;
        convert.unitList = yUnitsOut;
        convert.dataIn = str2double(SizeY);
        if strcmpi(dataType,'map'), convert.dataLabel = 'Y size'; else convert.dataLabel = 'Y Max Frequency'; end
        [OutCellArray,OutConfig] = UpdateHeader(OutCellArray,OutConfig,convert); % update value
        convert.dataIn = '';    % use default
        convert.dataLabel = 'Y unit';
        [OutCellArray,OutConfig] = UpdateHeader(OutCellArray,OutConfig,convert); % update unit label
        
        % convert Z-axis data and unit information
        convert.conversionFactor = conversionFactors.zUnit;
        convert.defaultLabel = UnitsZ;
        convert.pathList = zPathsOut;
        convert.unitList = zUnitsOut;
        convert.dataIn = DataZ;
        [OutCellArray,OutConfig] = UpdateData(OutCellArray,OutConfig,convert); % update value
        convert.dataIn = '';    % use default
        convert.dataLabel = 'Z unit';
        [OutCellArray,OutConfig] = UpdateHeader(OutCellArray,OutConfig,convert); % update unit label
        
        
        
    % ZERNIKE DATA TYPE    
    case 'zernike'
        % Read parameters and data
        SizeX=InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Radius'),2};
        DataZ=InCellArray{1,1}.data;
        UnitsX=ChangeUnitName(InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Radius unit'),2},'safe');
        UnitsZ=ChangeUnitName(InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Height unit'),2},'safe');
        
        % information common to all datasets
        figureArray.FontSize = defaultFontSize;
        figureArray.RecursionLimit = RECURSIONLIMIT;
        figureArray.dataType = dataType;
        
        % Create GUI elements for X-axis
        figureArray.Tag = 'radius';
        figureArray.Position = [0.2,0.6,0.25,0.1];
        figureArray.UnitsInput = UnitsX;
        figureArray.DataInput = SizeX;
        [xUnitsOut, xPathsOut, ~]=CreateGuiUnitDisplay(figureHandle,figureArray,InConfig);
                
        % Create GUI elements for height data (Z)
        figureArray.Tag = 'height';
        figureArray.Position = [0.5,0.6,0.25,0.1];
        figureArray.UnitsInput = UnitsZ;
        figureArray.DataInput = DataZ;
        [zUnitsOut, zPathsOut, ~]=CreateGuiUnitDisplay(figureHandle,figureArray,InConfig);
        
        %Create GUI Button to signal processing of all data 
        uicontrol('Parent',figureHandle,...
            'Tag','convertDataButton',...
            'Style','pushbutton',...
            'Enable','on',...
            'FontSize',defaultFontSize+1,...
            'FontWeight','normal',...
            'TooltipString','Apply unit conversions to all data..',...
            'Units','normalized',...  % must precede anything which uses dimensions
            'Position',[0.375,0.3,0.2,0.1],...
            'String','Convert Units',...
            'UserData','0',...
            'Callback',@GUIConvertData_Callback);
        
        % Perform actual conversion only when ready to exit (if not running in automated testing mode)
        if isfield(OutCellArray{1,1}, 'AutomatedMode')
            % Testing mode only
            parameters = OutCellArray{1,1}.AutomatedMode;
            allHandles = guihandles;
            set(allHandles.radiusUnitPopup,'Value',parameters(1));   % have user "select" desired unit conversion
            GUIUnit_Callback(allHandles.radiusUnitPopup,[],figureHandle,'radiusUnitPopup','radiusUnit');
            set(allHandles.heightUnitPopup,'Value',parameters(2));
            GUIUnit_Callback(allHandles.heightUnitPopup,[],figureHandle,'heightUnitPopup','heightUnit');
        else
            % Note: this wait interferes with automated testing, hence the check
            % to ensure the module isn't being run in automated mode.
            uiwait(figureHandle);
        end
        
        % Get conversion factors stored in main figure 'UserData'
        if ishandle(figureHandle),conversionFactors = get(figureHandle,'UserData');
        else return; % abort calculation without changes if user closes window
        end
        
        % convert X-axis unit information and scale
        convert.conversionFactor = conversionFactors.radiusUnit;
        convert.defaultLabel = UnitsX;
        convert.pathList = xPathsOut;
        convert.unitList = xUnitsOut;
        convert.dataIn = str2double(SizeX);
        convert.dataLabel = 'Radius';
        [OutCellArray,OutConfig] = UpdateHeader(OutCellArray,OutConfig,convert); % update value
        convert.dataIn = '';    % use default
        convert.dataLabel = 'Radius unit';
        [OutCellArray,OutConfig] = UpdateHeader(OutCellArray,OutConfig,convert); % update unit label

        % convert Z-axis data and unit information
        convert.conversionFactor = conversionFactors.heightUnit;
        convert.defaultLabel = UnitsZ;
        convert.pathList = zPathsOut;
        convert.unitList = zUnitsOut;
        convert.dataIn = '';    % use default
        convert.dataLabel = 'Height unit';
        [OutCellArray,OutConfig] = UpdateHeader(OutCellArray,OutConfig,convert); % update unit label
        convert.dataIn = DataZ(:,1);
        [OutCellArray,OutConfig] = UpdateData(OutCellArray,OutConfig,convert); % update value
        OutCellArray{1,1}.data(:,2:3) = InCellArray{1,1}.data(:,2:3);   % only update the first column of values since others are Zernike indices

        
        
        
    % COORDINATES DATA TYPE    
    case 'coordinates'
        % Read parameters and data
        LabelX = strtrim(regexp(InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Column Label'),2},',','split'));
        DataX = InCellArray{1,1}.data;
        UnitsX = strtrim(regexp(InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Column unit'),2},',','split'));
        for ii=1:length(UnitsX)
            UnitsX{ii} = ChangeUnitName(UnitsX{ii},'safe');
        end
        
        % information common to all datasets
        figureArray.FontSize = defaultFontSize;
        figureArray.RecursionLimit = RECURSIONLIMIT;
        figureArray.dataType = dataType;
        
        % Create GUI elements for columns
        xUnitsOut = cell(1,length(LabelX));
        xPathsOut = cell(1,length(LabelX));
        for ii = 1:length(LabelX)
            listBoxWidth = 0.9/length(LabelX)-0.02;
            listBoxXLocation = 0.05+(ii-1)*0.9/length(LabelX);
            figureArray.Tag = LabelX{ii};
            figureArray.Position = [listBoxXLocation,0.6,listBoxWidth,0.1];
            figureArray.UnitsInput = UnitsX{ii};
            figureArray.DataInput = DataX(:,ii);
            [xUnitsOut{ii}, xPathsOut{ii}, ~]=CreateGuiUnitDisplay(figureHandle,figureArray,InConfig);
        end
         
        %Create GUI Button to signal processing of all data 
        uicontrol('Parent',figureHandle,...
            'Tag','convertDataButton',...
            'Style','pushbutton',...
            'Enable','on',...
            'FontSize',defaultFontSize+1,...
            'FontWeight','normal',...
            'TooltipString','Apply unit conversions to all data..',...
            'Units','normalized',...  % must precede anything which uses dimensions
            'Position',[0.375,0.3,0.2,0.1],...
            'String','Convert Units',...
            'UserData','0',...
            'Callback',@GUIConvertData_Callback);
        
        % Perform actual conversion only when ready to exit (if not running in automated testing mode)
        if isfield(OutCellArray{1,1}, 'AutomatedMode')
            % Testing mode only
            parameters = OutCellArray{1,1}.AutomatedMode;
            allHandles = guihandles;
            for ii=1:length(LabelX);
                set(allHandles.([LabelX{ii},'UnitPopup']),'Value',parameters(1));   % have user "select" desired unit conversion
                GUIUnit_Callback(allHandles.([LabelX{ii},'UnitPopup']),[],figureHandle,[LabelX{ii},'UnitPopup'],[LabelX{ii},'Unit']);
            end
        else
            % Note: this wait interferes with automated testing, hence the check
            % to ensure the module isn't being run in automated mode.
            uiwait(figureHandle);
        end
        
        % Get conversion factors stored in main figure 'UserData'
        if ishandle(figureHandle),conversionFactors = get(figureHandle,'UserData');
        else return; % abort calculation without changes if user closes window
        end
        
        % convert data and unit information
        newLabel = cell(1,length(LabelX));
        newArray = nan(size(DataX));
        for ii = 1:length(LabelX)
            convert.conversionFactor = conversionFactors.([LabelX{ii},'Unit']);
            convert.defaultLabel = UnitsX{ii};
            convert.pathList = xPathsOut{ii};
            convert.unitList = xUnitsOut{ii};
            convert.dataIn = '';    % use default
            convert.dataLabel = 'Column unit';
            [OutCellArray,OutConfig] = UpdateHeader(OutCellArray,OutConfig,convert); % update unit label
            newLabel{ii} = OutCellArray{1}.header{strcmpi(OutCellArray{1}.header, 'Column unit'),2};
            if ii ~= length(LabelX), newLabel{ii} = [newLabel{ii},', ']; end
            convert.dataIn = DataX(:,ii);            
            [NewCellArray,OutConfig] = UpdateData(OutCellArray,OutConfig,convert); % update value
            newArray(:,ii) = NewCellArray{1,1}.data;
        end
        OutCellArray{1,1}.data = newArray;
        OutCellArray{1}.header{strcmpi(OutCellArray{1}.header, 'Column unit'),2} = [newLabel{:}];
        
        
        
        
    % UNKNOWN DATA TYPE    
    otherwise
        warndlg(['The data type given (',dataType,') is unsupported for this module. The data will not be modified ...'],'Unsupported data type!','modal');
end % end switch statement to process based on data type

% close the main figure if not running in automated mode
if ~isfield(OutCellArray{1,1}, 'AutomatedMode')
    % Note: this interferes with automated testing, hence the check
    % to ensure the module isn't being run in automated mode.
    delete(figureHandle);
end

end % end UnitConvert function and return

%% Determine available conversion units and computation path for conversion
function [availableUnits,computationPathIndices]=AvailableUnits(configList, unitIn, recursionDepth)
% AvailableUnits recursively searches for all available conversion units.
% Given a list of functional relations between units, this routine
% automatically determines all other possible connections (up to a
% specified recursion depth). Thus if cm -> m is defined and m -> nm is
% defined then this routine can connect cm -> nm even if not explicitely
% defined. If the number of steps reaches the recursion depth or the list of available 
% units does not change after a given step, then the search is ended.
%
% Notes: In general, the output unit may be derived by applying multiple 
% relationships sequentially. This means that if any one formula is
% incorrect, it may lead to non-obvious errors in related conversions.
%
% Inputs: 
%   configList: cell array of strings which list of functional relationships between units (example of one formula: {'mm','@(m)m*1000'} converts meters(m) to millimeters (mm)). See Matlab help on 'anonymous function'
%   unitIn: string indicating starting unit (example: 'mm')
%   recursionDepth [default = 5]: Maximum number of relations to make
%
% Outputs: [availableUnits,computationPathIndices]
%   availableUnits: 1-D cell array of strings functionally related to input unit
%   computationPathIndices: 2-D matrix of indices relating how each available unit is connected to the input unit. Each row corresponds to each output unit (zeros appear after end of path). Each index refers to a row in the configList (example: if row = [4,15,3] then apply formula #4 followed by formula #15 followed by formula #3 to convert between units.)
%
% Author: Greg Smith
% Email: gsmith@optics.arizona.edu
%
% Version history:
% 1/12/12 - Version 1 - G.S. - initial implementation
%
% [availableUnits,computationPathIndices] = AvailableUnits(configList, unitIn, recursionDepth)

% set max recursion depth if not provided
% note: if recursionDepth = 1, then only explicitly provided formulae will be used
if nargin < 3, recursionDepth = 5; end;

% get all defined units from cinfiguration list
allUnitsOut = configList(:,1);
allUnitsIn = regexp(configList(:,2),'\((\w+)\)','tokens','once');
allUnitsIn = [allUnitsIn{:,1}].';
functionCounter = transpose(1:length(allUnitsIn));  % list of indices to available relations/functions


availableUnits = [];            % container for output units
revisedUnits = {unitIn};        % unit list available at each step
computationPathIndices(1) = 0;  % container showing how each start unit (0) is functionally related to an end unit
recursionCounter = 1;           % keep track of how many loops through the relations
iterationFlag = true;           % flag to indicate when to break out of iteration
while iterationFlag
    
    % find all new units related to every unit at the current level of depth
    % update appropriate computation paths so conversion can occur
    currentComputationPaths = computationPathIndices;
    for ii = 1:length(revisedUnits)
        compareInput = strcmpi(allUnitsIn,revisedUnits(ii));
        revisedUnits = vertcat(revisedUnits,allUnitsOut(compareInput)); %#ok (G.S. - 1/11/2012 - end length of revisedUnits can not be known in general because it depends on the number of matches found and therefore cannot be pre-allocated)
        newComputationPaths = functionCounter(compareInput);
        currentComputationPath = currentComputationPaths(ii,:);
        for jj = 1:length(newComputationPaths)
            newPathArray = [currentComputationPath,newComputationPaths(jj)];
            computationPathIndices(size(computationPathIndices,1)+1,1:length(newPathArray)) = newPathArray; %#ok (G.S. - 1/12/2012 - end length of computationPaths can not be known in general because it depends on revisedUnits which is unknown length. computationsPaths variable therefore cannot be pre-allocated)
        end
    end
    
    % select unique units. This avoids duplications of final units. By 
    % using the 'first' option in the unique command, the shortest path to
    % end unit is used and avoids repetition such as mm -> m -> mm -> m as 
    % a path between mm and m
    % note: the unique command also sorts the units alphabetically
    [revisedUnits,uniqueLocations] = unique(revisedUnits,'first');
    computationPathIndices = computationPathIndices(uniqueLocations,:);
    
    % keep updating until recursion limit is reached or until the list of units does not change.
    if (recursionCounter <= recursionDepth) && (length(availableUnits) ~= length(revisedUnits))
        availableUnits = revisedUnits;
        recursionCounter = recursionCounter+1;
    else
        iterationFlag = false;
    end
end

% clean up path information by dropping extra zeros from start and end of formula path
% starting zeros used to denote initial unit and is redundant in final output
% ending zeros may appear if recursion stops due to no changes
computationPathIndices(:,1)=[];
if ~isempty(computationPathIndices) && sum(computationPathIndices(:,end)) == 0, computationPathIndices(:,end)=[]; end


% remove unit and path information associated with connecting input unit to itself
if isempty(computationPathIndices)
    % handle case where there is no connection to other units
    availableUnits = {};    
else
    [availableUnits,goodIndices] = setdiff(availableUnits,unitIn);
    computationPathIndices = computationPathIndices(goodIndices,:);
end

end

%% Create GUI text box and pulldown menu for display
function [unitsOut, pathsOut, displayUnits] = CreateGuiUnitDisplay(figureHandle,figureArray,availableUnitArray)
% CreateGuiUnitDisplay defines drop-down list boxes of available units.
% After basic processing to determine the available units, a text box and a
% drop-down list box are created in the figure given by figureHandls.
%
% Inputs: 
%   figureHandle: handle to figure where GUI elements should appear
%   figureArray: structure containing parameters to specify GUI properties
%   availableUnitArray: A list of all available units
%
% Outputs: [unitsOut, pathsOut, displayUnits]
%   unitsOut: 1-D cell array of strings functionally related to input unit
%   pathsOut: 2-D matrix of indices relating how each available unit is connected to the input unit. Each row corresponds to each output unit (zeros appear after end of path). Each index refers to a row in the configList (example: if row = [4,15,3] then apply formula #4 followed by formula #15 followed by formula #3 to convert between units.)
%   displayUnits: 1-D cell array of string with base value converted to appropriate units
%
% Author: Greg Smith
% Email: gsmith@optics.arizona.edu
%
% Version history:
% 1/13/12 - Version 1 - G.S. - initial implementation
%
% [unitsOut, pathsOut, displayUnits] = CreateGuiUnitDisplay(figureHandle,figureArray,availableUnitArray)

    % extract common parameters from figureArray structure
    RECURSIONLIMIT = figureArray.RecursionLimit;
    unitsInput = figureArray.UnitsInput;
    dataInput = figureArray.DataInput;
    dataType = figureArray.dataType;

    % get all available units
    [unitsOut,pathsOut] = AvailableUnits(availableUnitArray,unitsInput,RECURSIONLIMIT);

    % get a single value for display with converted units
    if ischar(dataInput)
        displayValue = str2double(dataInput);
    else
        displayValue = median(dataInput(~isnan(dataInput)));
    end

    % create list of converted units to display
    % first unit should be input unit to allow for no change in units
    % last two units should allow user to add new conversions
    displayUnits{length(unitsOut)+3}='Custom Function'; % pre-allocate array
    displayUnits{length(unitsOut)+2}='Custom Scale Factor';
    displayUnits{1} = [num2str(displayValue),' ',ChangeUnitName(unitsInput,'off')];
    for ii=1:length(unitsOut)    % compute converted value for each allowed unit
        value = displayValue;
        counter = 1;
        while counter <= length(pathsOut(ii,:)) && pathsOut(ii,counter) ~= 0
            func = str2func(availableUnitArray{pathsOut(ii,counter),2});
            value = func(value);
            counter = counter +1;
        end
        displayUnits{ii+1} = [num2str(value),' ',ChangeUnitName(unitsOut{ii},'off')];
    end
    
    % To allow for arbitrary functions or scaling factors, it is not enough
    % to know which element of the list box was selected. Therefore, the
    % current value of the selection is stored in the main figure 'UserData'
    dataLabel = [figureArray.Tag,'Unit'];
    userData = get(figureHandle,'UserData');
    userData.(dataLabel) = 1;
    set(figureHandle,'UserData',userData);    
    
    % create some elements for the GUI based on figureArray parameters
    if ischar(dataInput) && strcmpi(dataType,'map')
        textString = {[upper(figureArray.Tag),' Size: ',num2str(displayValue),' ',ChangeUnitName(unitsInput,'off'),];'';'Convert to:'};
    elseif ischar(dataInput)&& strcmpi(dataType,'freqmap')
        textString = {[upper(figureArray.Tag),' Max Frequency: ',num2str(displayValue),' ',ChangeUnitName(unitsInput,'off'),];'';'Convert to:'};
    elseif ischar(dataInput)
        textString = {[upper(figureArray.Tag),': ',num2str(displayValue),' ',ChangeUnitName(unitsInput,'off'),];'';'Convert to:'};
    else
        textString = {['Median ',upper(figureArray.Tag),' Data: ',num2str(displayValue),' ',ChangeUnitName(unitsInput,'off'),];'';'Convert to:'};
    end
    tagTextBox = [figureArray.Tag,'Unit'];
    tagPopup = [figureArray.Tag,'UnitPopup'];

    % add text box with current unit and other relevant information
    uicontrol('Parent', figureHandle, ...
        'Tag',tagTextBox,...
        'Style','text',...
        'Enable','on',...
        'HorizontalAlignment','left',...
        'FontSize',figureArray.FontSize,...
        'FontWeight','normal',...
        'Units','normalized',...  % must precede anything which uses dimensions
        'Position',figureArray.Position,...
        'String',textString);
    
    % add popup with available units to convert to
    uicontrol('Parent',figureHandle,...
        'Tag',tagPopup,...
        'Style','popupmenu',...
        'Enable','on',...
        'Value',1,... % default is first option (no unit conversion)
        'BackgroundColor','white',...
        'HorizontalAlignment','center',...
        'FontSize',figureArray.FontSize,...
        'FontWeight','normal',...
        'TooltipString','Select unit conversion',...
        'Units','normalized',...  % must precede anything which uses dimensions
        'Position',figureArray.Position-[0,0.1,0,0],...
        'String',displayUnits,...
        'Callback',{@GUIUnit_Callback,figureHandle,tagPopup,tagTextBox});
end

%% Callback for unit selection changes
function GUIUnit_Callback(hObject,~,handles,tagPopup,tagText)
% Callback function when a unit selection is changed.
% If one of the user-defined units is selected, a warning popup box is
% presented, and then information for the new unit is requested.
%
% Inputs: 
%   handles: handle to figure where GUI elements should appear
%   tagPopup: string indicator of the tag used to access the drop-down list dialog box
%   tagTest: string indicator of tag used to access the test field associated with each drop-down list box
%
% Outputs: N/A
%
% Author: Greg Smith
% Email: gsmith@optics.arizona.edu
%
% Version history:
% 1/13/12 - Version 1 - G.S. - initial implementation
%
% GUIUnit_Callback(hObject,~,handles,tagPopup,tagText)

    % get basic parameters stored elsewhere
    userData = guihandles(handles);
    unitList = get(userData.(tagPopup),'String');
    listLength = length(unitList);
    currentUnit = regexp(unitList{1},'\S+','match');
    selection = get(userData.(tagPopup),'Value');
    
    switch selection
        case listLength-1   % user selected arbitrary multiplicative scale factor
            % display warning
            warnHandle = warndlg({'This is inherently a dangerous operation! This new unit will be appended to your list of available conversion functions.','','Because unit conversions are cascaded to derive other units, any errors in this formula may cause subtle errors in other conversions.'},'Create Conversion Function ScaleFactor','modal');
            uiwait(warnHandle);
            
            % ask for new conversion factor
            dialogTitle = 'Convert Unit multiplicative scale factor';
            prompt = {'Enter name for new unit:',['Enter multiplicative scale factor (factor * ',currentUnit{2},' = newUnit):']};
            defaultAnswer = {'newUnit','1.0'};
            options.Resize='on';
            options.WindowStyle='modal';
            options.Interpreter='none';
            scaleFactor = inputdlg(prompt,dialogTitle,1,defaultAnswer,options);
            if isempty(scaleFactor)
                value = 0;
                set(userData.(tagPopup),'Value',1);
            else
                value = {scaleFactor{1},['@(',currentUnit{2},')',currentUnit{2},'*',scaleFactor{2}]};
                % update list of possible unit conversion factors
                currentListItems = get(hObject,'String');
                currentListItems{length(currentListItems)-1} = [currentListItems{length(currentListItems)-1},': ',scaleFactor{1},'=',currentUnit{2},'*',scaleFactor{2}];
                currentListItems{length(currentListItems)} = 'Custom Function';
                set(hObject,'String',currentListItems);
            end
            
        case listLength     % user selected arbitrary user function
            % display warning
            warnHandle = warndlg({'This is inherently a dangerous operation! This new unit will be appended to your list of available conversion functions.','','Because unit conversions are cascaded to derive other units, any errors in this formula may cause subtle errors in other conversions.'},'Create Arbitrary Unit Conversion Function','modal');
            uiwait(warnHandle);
            
            % ask for new conversion factor
            dialogTitle = 'Convert Unit arbitrary function';
            prompt = {'Convert TO: ',['enter anonymous function of the form: @(',currentUnit{2},')',currentUnit{2},' + 0']};
            defaultAnswer = {'unitTO',['@(',currentUnit{2},')',currentUnit{2},' + 0']};
            options.Resize='on';
            options.WindowStyle='modal';
            options.Interpreter='none';
            arbFunction = inputdlg(prompt,dialogTitle,1,defaultAnswer,options);
            if isempty(arbFunction)
                value = 0;
                set(userData.(tagPopup),'Value',1)
            else
                value = arbFunction;
                % update list of possible unit conversion factors
                currentListItems = get(hObject,'String');
                currentListItems{length(currentListItems)} = [currentListItems{length(currentListItems)},': ',value{1},'=',value{2}];
                currentListItems{length(currentListItems)-1} = 'Custom Scale Factor';
                set(hObject,'String',currentListItems);
            end
            
        otherwise  % user selected a standard unit conversion
            value = selection;
    end
    
    % update figure 'UserData' value with current selection information
    userData = get(handles,'UserData');
    userData.(tagText) = value;
    set(handles,'UserData',userData);
end

%% Callback for button press to signal conversion and exit
function GUIConvertData_Callback(~,~)
% Callback function when the button is pressed indicating ready for conversion.
%
% Inputs: Standard Callback inputs
%
% Outputs: N/A
%
% Author: Greg Smith
% Email: gsmith@optics.arizona.edu
%
% Version history:
% 1/13/12 - Version 1 - G.S. - initial implementation
%
% GUIConvertData_Callback(hObject,~)

    % update the magic flag so we can get out of here!
    uiresume(gcbf);
end

%% Update configuration information
function OutConfig=UpdateConfig(InConfig,newCell)
% Update Configuration parameters by appending the new cell
%
% Inputs: 
%    InConfig: SAGUARO standard configuration list
%    newCell: information same as rown of standard SAGUARO configuration list
%
% Outputs: OutConfig
%    OutConfig: SAGUARO standard configuration list
%
% Author: Greg Smith
% Email: gsmith@optics.arizona.edu
%
% Version history:
% 1/13/12 - Version 1 - G.S. - initial implementation
%
% OutConfig=UpdateConfig(InConfig,newCell)

    % use input as default output
    OutConfig = InConfig;

    % add new unit to OutConfig
    OutConfig{length(OutConfig)+1,1} = newCell{1};
    OutConfig{length(OutConfig),2} = newCell{2};
end

%% Update header information in cell array
function [OutCellArray,OutConfig]=UpdateHeader(InCellArray,InConfig,convertStruct)
% Update SAGUARO standard header parameters with new unit values and labels
%
% Inputs:
%    InCellArray: SAGUARO standard cell array
%    InConfig: SAGUARO standard configuration list
%    convertStruct: structure containing information necessary for the conversion including which elements of the cellArray.header are  modified
%
% Outputs: [OutCellArray,OutConfig]
%    OutCellArray: SAGUARO standard cell array
%    OutConfig: SAGUARO standard configuration list
%
% Author: Greg Smith
% Email: gsmith@optics.arizona.edu
%
% Version history:
% 1/13/12 - Version 1 - G.S. - initial implementation
%
% [OutCellArray,OutConfig]=UpdateHeader(InCellArray,InConfig,convertStruct)

    % use input as default output
    OutCellArray = InCellArray;
    OutConfig = InConfig;
    
    % extract information from convert structure
    data = convertStruct.dataIn;
    headerName = convertStruct.dataLabel;
    conversion = convertStruct.conversionFactor;
    defaultLabel = convertStruct.defaultLabel;
    paths = convertStruct.pathList;
    units = convertStruct.unitList;
    
    % act based on type of conversion requested
    if iscell(conversion)
        OutConfig = UpdateConfig(OutConfig,conversion);
        conversionFactor = length(OutConfig);
        unitLabel = ChangeUnitName(OutConfig{length(OutConfig),1},'off');
    elseif conversion == 1
        conversionFactor = 'none';  % do not update if no unit change
        unitLabel = ChangeUnitName(defaultLabel,'off');
    else
        conversionFactor = paths(conversion-1,:);
        unitLabel = ChangeUnitName(units{conversion-1,1},'off');
    end
    if ~isnumeric(data), data = unitLabel; end
    
    % convert data
    newData = data;
    if ~ischar(conversionFactor) && isnumeric(newData)
        counter = 1;
        while counter <= length(conversionFactor) && conversionFactor(counter) > 0
            func = str2func(OutConfig{conversionFactor(counter),2});
            newData = func(newData);
            counter = counter +1;
        end
    end
    if ~ischar(newData), newData = num2str(newData); end
    
    % update OutCellArray
    OutCellArray{1}.header{strcmpi(OutCellArray{1}.header, headerName),2} = newData;
end

%% Update data
function [OutCellArray,OutConfig]=UpdateData(InCellArray,InConfig,convertStruct)
% Update SAGUARO cell array with new data
%
% Inputs:
%    InCellArray: SAGUARO standard cell array
%    InConfig: SAGUARO standard configuration list
%    convertStruct: structure containing information necessary for the conversion.
%
% Outputs: [OutCellArray,OutConfig]
%    OutCellArray: SAGUARO standard cell array
%    OutConfig: SAGUARO standard configuration list
%
% Author: Greg Smith
% Email: gsmith@optics.arizona.edu
%
% Version history:
% 1/13/12 - Version 1 - G.S. - initial implementation
%
% [OutCellArray,OutConfig]=UpdateData(InCellArray,InConfig,convertStruct)

    % use input as default output
    OutCellArray = InCellArray;
    OutConfig = InConfig;
    
    % extract information from convert structure
    data = convertStruct.dataIn;
    conversion = convertStruct.conversionFactor;
    paths = convertStruct.pathList;
    
    % act based on type of conversion requested
    if iscell(conversion)
        OutConfig = UpdateConfig(OutConfig,conversion);
        conversionFactor = length(OutConfig);
    elseif conversion == 1
        conversionFactor = 'none';  % do not update if no unit change
    else
        conversionFactor = paths(conversion-1,:);
    end
    
    % convert data
    newData = data;
    if ~ischar(conversionFactor) && isnumeric(newData)
        counter = 1;
        while counter <= length(conversionFactor) && conversionFactor(counter) > 0
            func = str2func(OutConfig{conversionFactor(counter),2});
            newData = func(newData);
            counter = counter +1;
        end
    end
    
    % update OutCellArray
    OutCellArray{1,1}.data = newData;
end

%% Handle unit names with atypical characters
function unitOut=ChangeUnitName(unitIn,safeMode)
% Convert unit names with non-standard (non-ASCII) names.
% Several units such as m use special characters such as the mu character.
% Although the unit-conversion functions can not use these special
% characters, it is nice to display them. This routine maintains a list of
% conversion factors and will change the input unit to its partner form.
%
% Inputs: 
%    unitIn: input unit (string)
%    safeMode: indicator whether special characters should be allowed in output (string) [default = 'off']
%
% Outputs: OutConfig
%    unitOut: output unit (string)
%
% Author: Greg Smith
% Email: gsmith@optics.arizona.edu
%
% Version history:
% 1/13/12 - Version 1 - G.S. - initial implementation
%
% unitOut=ChangeUnitName(unitIn,safeMode)

    % input sanitization
    if nargin < 2, safeMode = 'off'; end
    if ~ischar(safeMode), safeMode = num2str(safeMode); end

    convertList={
        'um','m';
        
        'invm','1/m';
        'invcm','1/cm';
        'invmm','1/mm';
        'invum','1/m';
        'invum','1/um';
        'invnm','1/nm';
        
        'urad','rad';
        
        'us','s';
        
        'degF','F';
        'degC','C';
        
        'uW','W';
        };
    
    % find all matches
    locations=strcmpi(convertList,unitIn);
    
    % convert left-right paris only if necessary
    if isequal(locations,false(size(locations)))
        % do not convert unit if no matches found
        unitOut = unitIn;
    elseif sum(locations(:,2))==0 && (strcmpi(safeMode,'safe')||strcmpi(safeMode,'true')||strcmpi(safeMode,'on'))
        % do not convert unit in 'safe' mode unless input unit is initially on right-hand side of convertList
        unitOut = unitIn;
    else
        unitOut=convertList{fliplr(locations)};
    end
end