function [OutCellArray, Outconfig]=Zernike2Map(InCellArray, Inconfig)
%Zernike2Map converts a set of zernike coefficients into a map.  The
%coefficients are sorted into ZEMAX order, and then the polynomials are
%recursively generated, and the map computed.
%
%Inputs: ZernikeSTD or ZernikeAnnular
%Outputs: Map
%
%Author: Benjamin Lewis
%Email: blewis@email.arizona.edu
%
% Version history:
% 10/30/11 - Version 2
%
% The Terms of Use may be found at the end of this code.




%Create header
OutCellArray{1}.header={'Date Created', ''};

Radius=InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Radius'),2};

OutCellArray{1}.header(end+1,:)={'X size', Radius};
OutCellArray{1}.header(end+1,:)={'Y size', Radius};

R_unit=InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Radius unit'),2};

OutCellArray{1}.header(end+1,:)={'X unit', R_unit};
OutCellArray{1}.header(end+1,:)={'Y unit', R_unit};

H_unit=InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Height unit'),2};

OutCellArray{1}.header(end+1,:)={'Z unit', H_unit};


OutCellArray{1}.type='map';

Outconfig=Inconfig;


ZernRemove=Inconfig{strcmpi(Inconfig, 'Terms Removed'),2};

pts=Inconfig{strcmpi(Inconfig, 'Data Points'),2};

validinputs=0;

%Get inptuts
while ~validinputs
    answers = inputdlg({'Number of Points:','Terms to Remove (ex: 1,2,3,4)'},'Zernike2map',1,{pts, ZernRemove});
    if ~isempty(answers)
        temppts=answers{1};        
        temppts=str2num(temppts);
        
        tempterms=answers{2};      
        expr='(\d)*,|(\d)*';
        tempscell=regexp(tempterms,expr,'tokens');
        tempterms=[];
        for kk=1:length(tempscell)
            temp=floor(str2num(char(tempscell{kk})));
            if ~isempty(temp)&&isnumeric(temp)
                tempterms(end+1)=temp;
            end
        end

        if ~isempty(temppts)&&isnumeric(temppts)&&isempty(tempterms)
            validinputs=1;           
            pts=temppts;
            ZernRemove=[];
        elseif ~isempty(temppts)&&isnumeric(temppts)&&~isempty(tempterms)&&isnumeric(tempterms)
            validinputs=1;
            pts=temppts;
            ZernRemove=tempterms;
        else
            msgbox('Please enter valid inputs','Invalid Inputs','warn','modal')
        end
    else
        OutCellArray={};
        return; %If answers is empty, user canceled
    end
end

Outconfig{strcmpi(Outconfig, 'Data Points'),2}=num2str(pts);

outstring=sprintf('%i, ', ZernRemove);

Outconfig{strcmpi(Outconfig, 'Terms Removed'),2}=outstring(1:end-2);


epsilon=InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Obscuration Ratio'),2};

epsilon=str2num(epsilon);
        

zern_coef=InCellArray{1}.data;

coefs=sort_and_zero_zerns(zern_coef, ZernRemove);


x=linspace(-1, 1, pts);
x=meshgrid(x);
[theta, rho] = cart2pol(x,-1*x');

%pixeltolerance=.5;
pixeltolerance=0;

theta(rho>(pts+pixeltolerance)/pts)=NaN; %do radius edging with a half-pixel tolerance

rho(rho>(pts+pixeltolerance)/pts)=NaN;

dmap=ComputeMap(rho, theta, epsilon, coefs);


OutCellArray{1}.data=dmap;

    function coefs=sort_and_zero_zerns(zern_coef, ZernRemove)
        %Adapted from zStd_Example
        % Author: Ping Zhou       
        
        zern_coef(:,2)=floor(zern_coef(:,2));
        zern_coef(:,3)=floor(zern_coef(:,3));
        
        minTerm=1;
        maxN=max(zern_coef(:,3));  
        maxTerm=sum(1:maxN+1); %Calculate max term based on zernike tree        
        
        m=0;
        n=0;
        jj=1;
        indices(1,:)=[0,0];
        
        %Generate indeces in ZEMAX order, for use in sorting
        while jj<maxTerm
            if m==n||m==-1*n
                n=n+1;
                if mod(n,2)==0, m=0; else m=1; end
            else
                m=m+2;
            end
            if m==0
                indices(jj+1,:)=[m, n]; %We can't index to 0, so zernike z0 maps to 1
            else
                if mod(jj,2)~=0 %Strangely enough, ZEMAX uses this to decide the order of terms
                    indices(jj+1,:)=[m, n];
                    jj=jj+1;
                    indices(jj+1,:)=[-1*m, n];
                else
                    indices(jj+1,:)=[-1*m, n];
                    jj=jj+1;
                    indices(jj+1,:)=[m, n];
                end
            end
            
            jj=jj+1;
        end
        
        indices=indices(1:maxTerm,:);
        
        coefs=zeros(maxTerm, 1);
        
        %Sort zernike coefficients into ZEMAX order
        for ii=1:maxTerm
            if ~isempty(zern_coef(zern_coef(:,2)==indices(ii,1)&zern_coef(:,3)==indices(ii,2)))
                coefs(ii)=zern_coef(zern_coef(:,2)==indices(ii,1)&zern_coef(:,3)==indices(ii,2));
            else
                coefs(ii)=0;
            end
        end
        
        %while coefs(end)==0, coefs(end)=[]; end
        
        for tt = minTerm:maxTerm
            if any(tt==ZernRemove)
                coefs(tt)=0;
            end
        end
    end
        


    function [dmap]=ComputeMap(rho, theta, epsilon, coefs)        
        maxTerm=length(coefs);
        
        m=0;
        n=0;
        
        tt=1;
        indices(1,:)=[0,0,0];
        
        while tt<maxTerm
            if m==n||m==-1*n
                n=n+1;
                if mod(n,2)==0, m=0; else m=1; end
            else
                m=m+2;
            end
            if m==0
                indices(tt+1,:)=[m, n, (n-abs(m))/2]; %We can't index to 0, so zernike z0 maps to 1
            else
                if mod(tt,2)~=0 %Strangely enough, ZEMAX uses this to decide the order of terms
                    indices(tt+1,:)=[m, n, (n-abs(m))/2];
                    tt=tt+1;
                    indices(tt+1,:)=[-1*m, n, (n-abs(m))/2];
                else
                    indices(tt+1,:)=[-1*m, n, (n-abs(m))/2];
                    tt=tt+1;
                    indices(tt+1,:)=[m, n, (n-abs(m))/2];
                end
            end
            
            tt=tt+1;
        end
        
        indices=indices(1:length(coefs),:);
        
        if size(indices,1)>1
            Maxs=max(abs(indices));
            MaxM=Maxs(1);
            MaxJ=Maxs(3);
        else
            MaxM=abs(indices(1));
            MaxJ=abs(indices(3));
        end
        
        ThisRow=cell(1,MaxM+MaxJ-1); %This is going the Q Polynomials
        LastRow=cell(1,MaxM+MaxJ-1); %This is going the Q Polynomials
        
        thisH=zeros(1,MaxM+MaxJ-1);
        lastH=zeros(1,MaxM+MaxJ-1);
        
        [ysize,xsize]=size(rho);
        
        rho=rho(:);
        theta=theta(:);       
        
        rho(end+1)=0;
        theta(end+1)=0;

        
        X=(2.*((rho.^2-epsilon^2)./(1-epsilon^2)))-1;
        X(isnan(rho))=NaN;
        
        dmap=zeros(xsize,ysize);
        
        for mm=0:MaxM
            Qsum=zeros(size(rho));
            for jj=0:MaxM+MaxJ-1-(mm-1)
                if mm==0
                    if jj==0
                        TempQ=ones(size(rho));
                        TempQ(isnan(rho))=NaN;
                        ThisRow{jj+1}=TempQ;
                        thisH(jj+1)=(1-epsilon^2)/(2*(2*jj+1));
                    elseif jj==1
                        ThisRow{jj+1}=X;
                        thisH(jj+1)=(1-epsilon^2)/(2*(2*jj+1));
                    else
                        ThisRow{jj+1}=(2*(jj-1)+1).*X.*ThisRow{jj}-(jj-1).*ThisRow{jj-1};
                        ThisRow{jj+1}=ThisRow{jj+1}./((jj-1)+1);
                        thisH(jj+1)=(1-epsilon^2)/(2*(2*jj+1));
                    end
                else
                    a=2*(2*jj+2*mm-1)*lastH(jj+1);
                    b=(jj+mm)*(1-epsilon^2)*LastRow{jj+1}(end);
                    c=a/b;
                    
                    Qsum=Qsum+LastRow{jj+1}.*LastRow{jj+1}(end)./lastH(jj+1);
                    
                    ThisRow{jj+1}=c.*Qsum;
                    
                    a=2*(2*jj+2*mm-1)*LastRow{jj+2}(end);
                    b=(jj+mm)*(1-epsilon^2)*LastRow{jj+1}(end);
                    
                    thisH(jj+1)=-1*(a/b)*lastH(jj+1);
                end
            end
            
            LastRow=ThisRow;
            lastH=thisH;
            
            rhoM=rho.^mm;
            
            for ii=1:size(indices,1)
                if abs(indices(ii,1))==mm
                    m=indices(ii,1);
                    n=indices(ii,2);
                    j=indices(ii,3);
                    
                    scale=(1-epsilon^2)/(2*(2*j+mm+1)*thisH(j+1));
                    scale=scale^.5;
                    scale=scale.*rhoM;
                    
                    scale(rho<(epsilon-.5/size(rho,1)))=NaN;
                    if m==0 %Multiply by cos or sin if nessicary, and normalize so RMS=1
                        zernike=sqrt(n+1).*scale.*ThisRow{j+1};
                    elseif m>0
                        zernike=sqrt(2)*sqrt(n+1).*scale.*ThisRow{j+1}.*cos(mm.*theta);
                    else
                        zernike=sqrt(2)*sqrt(n+1).*scale.*ThisRow{j+1}.*sin(mm.*theta);
                    end                  
                    
                    zernike(end)=[];
                    
                    dmap=dmap+coefs(ii).*reshape(zernike,ysize,xsize);
                end
            end
        end        
    end
end

